@@ -29,14 +29,14 @@ class HistoDataset(torch.utils.data.Dataset):
         pre_transfs: O[U[TransformerHandler, Callable, Sequence[Callable]]] = None,
         da_transfs: O[U[TransformerHandler, Callable, Sequence[Callable]]] = None,
         seed: int = None,
-        return_data_rows: bool = False,
+        return_index: bool = False,
         return_pytorch_tensor: bool = True,
     ):
         super().__init__()
 
         # save variables
         self.path_to_dataset = path_to_dataset
-        self.return_data_rows = return_data_rows
+        self.return_index = return_index
         self.seed = seed
         self.seed_increment = 0
         self.return_pytorch_tensor = return_pytorch_tensor
@@ -96,60 +96,13 @@ class HistoDataset(torch.utils.data.Dataset):
         self.preloaded_data = None
         self.preloaded_features = None
 
-    def get_feature_for_all_rows(self, feature_pos=None):
-        if self.preloaded_data is not None:
-            if feature_pos is None:
-                return self.preloaded_features
-            else:
-                return [row[feature_pos] for row in self.preloaded_features]
-        else:
-            if feature_pos is None:
-                return [
-                    feature_loader(self.df, self.path_to_dataset)
-                    for feature_loader in self.feature_readers
-                ]
-            else:
-                return self.feature_readers[feature_pos](self.df, self.path_to_dataset)
-
-    """
-
     def preload(self, batch_size=128):
         self.preloaded_data = {}
         self.preloaded_features = {}
         for i in range(0, len(self.df), batch_size):
-            img, feat = self.__load__(self.df.iloc[i : i + batch_size].index)
-            for k in img:
-                if isinstance(img[k], list):
-                    img[k] = np.array(img[k])
-                if k in self.preloaded_data:
-                    if isinstance(img[k], np.ndarray):
-                        self.preloaded_data[k] = np.concatenate([self.preloaded_data[k], img[k]])
-                    else:
-                        self.preloaded_data[k] = torch.cat([self.preloaded_data[k], img[k]], 0)
-                else:
-                    self.preloaded_data[k] = img[k]
+            data = self.__load_data__(self.df.iloc[i : i + batch_size].index)
+            features = self.__load_features__(self.df.iloc[i : i + batch_size].index)
 
-            for k in feat:
-                if isinstance(feat[k], list):
-                    feat[k] = np.array(feat[k])
-                if k in self.preloaded_features:
-                    if isinstance(feat[k], np.ndarray):
-                        self.preloaded_features[k] = np.concatenate(
-                            [self.preloaded_features[k], feat[k]]
-                        )
-                    else:
-                        self.preloaded_features[k] = torch.cat(
-                            [self.preloaded_features[k], feat[k]], 0
-                        )
-                else:
-                    self.preloaded_features[k] = feat[k]
-    """
-
-    def preload(self, batch_size=128):
-        self.preloaded_data = {}
-        self.preloaded_features = {}
-        for i in range(0, len(self.df), batch_size):
-            data, features = self.__load__(self.df.iloc[i : i + batch_size].index)
             for k in data:
                 if k in self.preloaded_data:
                     self.preloaded_data[k].extend(data[k])
@@ -157,12 +110,12 @@ class HistoDataset(torch.utils.data.Dataset):
                     self.preloaded_data[k] = data[k]
 
             for k in features:
-                if k in self.preloaded_data:
+                if k in self.preloaded_features:
                     self.preloaded_features[k].extend(features[k])
                 else:
                     self.preloaded_features[k] = features[k]
 
-    def __load__(self, idx):
+    def __load_data__(self, idx):
         # get asked row
         row = self.df.iloc[idx]
 
@@ -175,15 +128,22 @@ class HistoDataset(torch.utils.data.Dataset):
                     data[key + "_" + k] = loaded[k]
             else:
                 data[key] = loaded
+
+        # pre transform
+        data = self.pre_transfs(data, self.__get_hash__(idx))
+
+        return data
+
+    def __load_features__(self, idx):
+        # get asked row
+        row = self.df.iloc[idx]
+
         features = {
             key: feature_loader(row, self.path_to_dataset)
             for key, feature_loader in self.feature_readers.items()
         }
 
-        # pre transform
-        data = self.pre_transfs(data, self.__get_hash__(idx))
-
-        return data, features
+        return features
 
     def __len__(self):
         return len(self.df)
@@ -196,24 +156,74 @@ class HistoDataset(torch.utils.data.Dataset):
             hashes = hashes.values.tolist()
         return hashes
 
-    def __getitem__(self, idx):
+    def __convert_to_tensor__(self, data, idx):
+        for key in data:
+            if isinstance(idx, list):
+                if isinstance(data[key], list):
+                    if not torch.is_tensor(data[key][0]):
+                        for i, d in enumerate(data[key]):
+                            if isinstance(d, np.ndarray):
+                                data[key][i] = torch.as_tensor(d.copy())
+                            else:
+                                data[key][i] = torch.as_tensor(d)
+                    data[key] = torch.stack(data[key])
+                elif not torch.is_tensor(data[key]):
+                    data[key] = torch.as_tensor(data[key])
+            elif not torch.is_tensor(data[key]):
+                if isinstance(data[key], np.ndarray):
+                    data[key] = torch.as_tensor(data[key].copy())
+                else:
+                    data[key] = torch.as_tensor(data[key])
+        return data
 
+    def get_features(self, idx):
         if isinstance(idx, np.ndarray):
             idx = list(idx)
 
-        # get asked row
-        row = self.df.iloc[idx]
+        if self.preloaded_data is None:
+            features = self.__load_features__(idx)
+        else:
+            features = {}
+            for k in self.preloaded_features:
+                if isinstance(idx, list):
+                    features[k] = [self.preloaded_features[k][_idx] for _idx in idx]
+                else:
+                    features[k] = self.preloaded_features[k][idx]
+        if self.return_pytorch_tensor:
+            features = self.__convert_to_tensor__(features, idx)
+
+        return features
+
+    def get_feature_for_all_rows(self):
+        if self.preloaded_data is not None:
+            return self.preloaded_features
+        else:
+            return self.get_features(np.arange(len(self)))
+
+    def __getitem__(self, idx):
+        if isinstance(idx, np.ndarray):
+            idx = list(idx)
 
         if self.seed is not None:
             # save random state to reactivate this state after the call
             saved_seed_state = helper.get_random_seed()
             # set seets
-            helper.set_random_seed_with_int((1 + self.seed_increment) * int(1e9) + self.seed)
+            worker_info = torch.utils.data.get_worker_info()
+            if worker_info is None:
+                worker_id = 0
+                num_workers = 1
+            else:
+                worker_id = worker_info.id
+                num_workers = worker_info.num_workers
+            seed_to_set = (1 + self.seed_increment + worker_id) * int(1e7) + self.seed
+
+            helper.set_random_seed_with_int(seed_to_set)
             # increase seed increment to use a new random seed at next call
-            self.seed_increment += 1
+            self.seed_increment += num_workers
 
         if self.preloaded_data is None:
-            data, features = self.__load__(idx)
+            data = self.__load_data__(idx)
+            features = self.__load_features__(idx)
         else:
             data = {}
             for k in self.preloaded_data:
@@ -236,45 +246,12 @@ class HistoDataset(torch.utils.data.Dataset):
             helper.set_random_seed(*saved_seed_state)
 
         if self.return_pytorch_tensor:
-            for key in data:
-                if isinstance(idx, list):
-                    if isinstance(data[key], list):
-                        if not torch.is_tensor(data[key][0]):
-                            for i, d in enumerate(data[key]):
-                                if isinstance(d, np.ndarray):
-                                    data[key][i] = torch.as_tensor(d.copy())
-                                else:
-                                    data[key][i] = torch.as_tensor(d)
-                        data[key] = torch.stack(data[key])
-                    elif not torch.is_tensor(data[key]):
-                        data[key] = torch.as_tensor(data[key])
-                elif not torch.is_tensor(data[key]):
-                    if isinstance(data[key], np.ndarray):
-                        data[key] = torch.as_tensor(data[key].copy())
-                    else:
-                        data[key] = torch.as_tensor(data[key])
-
-            for key in features:
-                if isinstance(idx, list):
-                    if isinstance(features[key], list):
-                        if not torch.is_tensor(features[key][0]):
-                            for i, d in enumerate(features[key]):
-                                if isinstance(d, np.ndarray):
-                                    features[key][i] = torch.as_tensor(d.copy())
-                                else:
-                                    features[key][i] = torch.as_tensor(d)
-                        features[key] = torch.stack(features[key])
-                    elif not torch.is_tensor(features[key]):
-                        features[key] = torch.as_tensor(features[key])
-                elif not torch.is_tensor(features[key]):
-                    if isinstance(features[key], np.ndarray):
-                        features[key] = torch.as_tensor(features[key].copy())
-                    else:
-                        features[key] = torch.as_tensor(features[key])
+            data = self.__convert_to_tensor__(data, idx)
+            features = self.__convert_to_tensor__(features, idx)
 
         dic = {}
         dic.update(data)
         dic.update(features)
-        if self.return_data_rows:
-            dic["raw"] = row
+        if self.return_index:
+            dic["idx"] = idx
         return dic
