@@ -76,7 +76,7 @@ class TransformerHandler:
             self.is_callable_with_batches[index] = trans.is_callable_with_batches
         else:
             self.is_callable_with_batches[index] = is_callable_with_batches
-
+  
     def __call_single_transform_with_dict__(
         self, item_or_batch, hashes, transf, dictionary, batch_callable
     ):
@@ -94,17 +94,12 @@ class TransformerHandler:
             item_or_batch.update(result)
             return item_or_batch
         else:
-            """
-            result = {}
             for i in range(len(item_or_batch[list(item_or_batch)[0]])):
-                if self.seed is not None:
-                    helper.set_random_seed_with_int(int(self.seed + hashes[j]))
-                element
-                input_dic = __set_values_to_input_dictionary__(dictionary, item_or_batch)
-                result = transf(**input_dic)
-                item_or_batch.update(result)
-            """
-            raise NotImplementedError()
+                item = {k:item_or_batch[k][i] for k in item_or_batch}
+                item = self.__call_single_transform_with_dict__(item, hashes[i], transf, dictionary, batch_callable)
+                for k in item:
+                    item_or_batch[k][i] = item[k]
+            return item_or_batch
 
     def __call_single_transform_with_callable__(
         self, item_or_batch, hashes, transf, order, batch_callable
