@@ -13,6 +13,7 @@ def mai_patches(
     transformation: O[U[Callable, Sequence[Callable]]] = None,
     pre_transformation: O[U[Callable, Sequence[Callable]]] = None,
     folds: O[Sequence[int]] = None,
+    subset: O[str] = None,
     gt_mode: str = "GT3&GT2",
     stain_mode: str = "phh3",
     seed: O[int] = None,
@@ -37,6 +38,12 @@ def mai_patches(
         folds (list(int)): The dataset is splitted into 10 folds. The
                        last (the 9th) should be used only for testing.
                        The list contains numbers between 0 and 9.
+        subset: which part of the dataset should be used.
+                       One of {"train", "valid", "test", "all"}
+                       This parameter is only used if folds is not set.
+                       "train" equals folds: [0,1,2,3,4,5,6],
+                       "valid" equals folds: [7,8],
+                       "test" equals folds: [9]
         gt_mode (str): The pathologist take a look at the slides three times
                        (GT1, GT2, GT3). One of the following modes are allowed:
                        - "GT1": 'None-Mitosis' / 'Mitosis' from the first study
@@ -84,6 +91,18 @@ def mai_patches(
                       as normal pytorch dataset. If you call it you will get
                       a dictionary back.
     """
+    if folds is None and subset is not None:
+        if subset == "train":
+            folds = [0, 1, 2, 3, 4, 5, 6]
+        elif subset == "val":
+            folds = [7, 8]
+        elif subset == "test":
+            folds = [9]
+        else:
+            raise AttributeError(
+                "Such type is not supported. Please choose the subset 'train', 'val' or 'test'."
+            )
+
     if folds is not None:
         manipulators_to_use = [df_manipulators.ColumnFilter("fold", folds)]
     else:
